// custom structures extracted from document
// Microsoft Portable Executable and Common Object File Format Specification.doc
// http://www.microsoft.com/whdc/system/platform/firmware/PECOFF.mspx

//////////////////////////////
// OBJ file
#define COFF_OBJHEADER_SIZE 20
struct  COFF_OBJHEADER,1
{
	unsigned word Machine;
	unsigned word NumberOfSections; // PointerToSectionTable = len(COFF_OBJHEADER)
	unsigned int  TimeDateStamp;
	unsigned int  OffsetToSymbolTable;
	unsigned int  NumberOfSymbols;
	unsigned word SizeOfOptionalHeader;
	unsigned word Characteristics;
	// COFF_SECTIONTABLE
}
#define COFF_SECTIONTABLE_SIZE 40
struct  COFF_SECTIONTABLE,1
{
	dstring Name[8]; // not always NULL terminated; if name="/int" - name is in COFF_STRINGTABLE+int
	unsigned int  VirtualSize;    // This field is valid only for executable images and should be set to 0 for object files
	unsigned int  VirtualAddress; // RVA/Offset
	unsigned int  SizeOfRawData;
	unsigned int  PointerToRawData;
	unsigned int  PointerToRelocations;
	unsigned int  PointerToLinenumbers;
	unsigned word NumberOfRelocations;
	unsigned word NumberOfLinenumbers;
	unsigned int  Characteristics;
}
// Section Number Values
#define IMAGE_SYM_UNDEFINED	0x00000000
#define IMAGE_SYM_ABSOLUTE	0xFFFFFFFF
#define IMAGE_SYM_DEBUG		0xFFFFFFFE

#define COFF_RELOCATIONS_SIZE 10
struct  COFF_RELOCATIONS,1
{
	unsigned int  VirtualAddress;  //Address of the item to which relocation is applied: this is the offset from the beginning of the section, plus the value of the sections RVA/Offset field (see Section 4, Section Table.). For example, if the first byte of the section has an address of 0x10, the third byte has an address of 0x12.
	unsigned int  SymbolTableIndex;//A zero-based index into the symbol table. This symbol gives the address to be used for the relocation. If the specified symbol has section storage class, then the symbols address is the address with the first section of the same name
	unsigned word Type;
}
// .Type  Intel 386
#define IMAGE_REL_I386_ABSOLUTE	0x0000	//This relocation is ignored.
#define IMAGE_REL_I386_DIR16	0x0001	//Not supported.
#define IMAGE_REL_I386_REL16	0x0002	//Not supported.
#define IMAGE_REL_I386_DIR32	0x0006	//The targets 32-bit virtual address.
#define IMAGE_REL_I386_DIR32NB	0x0007	//The targets 32-bit relative virtual address.
#define IMAGE_REL_I386_SEG12	0x0009	//Not supported.
#define IMAGE_REL_I386_SECTION	0x000A	//The 16-bit-section index of the section containing the target. This is used to support debugging information.
#define IMAGE_REL_I386_SECREL	0x000B	//The 32-bit offset of the target from the beginning of its section. This is used to support debugging information as well as static thread local storage.
#define IMAGE_REL_I386_REL32	0x0014	//The 32-bit relative displacement to the target. This supports the x86 relative branch and call instructions.
// .Type  MIPS Processors
#define IMAGE_REL_MIPS_ABSOLUTE		0x0000	//This relocation is ignored.
#define IMAGE_REL_MIPS_REFHALF		0x0001	//The high 16 bits of the targets 32-bit virtual address.
#define IMAGE_REL_MIPS_REFWORD		0x0002	//The targets 32-bit virtual address.
#define IMAGE_REL_MIPS_JMPADDR		0x0003	//The low 26 bits of the targets virtual address. This supports the MIPS J and JAL instructions.
#define IMAGE_REL_MIPS_REFHI		0x0004	//The high 16 bits of the targets 32-bit virtual address. Used for the first instruction in a two-instruction sequence that loads a full address. This relocation must be immediately followed by a PAIR relocations whose SymbolTableIndex contains a signed 16-bit displacement which is added to the upper 16 bits taken from the location being relocated.
#define IMAGE_REL_MIPS_REFLO		0x0005	//The low 16 bits of the targets virtual address.
#define IMAGE_REL_MIPS_GPREL		0x0006	//16-bit signed displacement of the target relative to the Global Pointer (GP) register.
#define IMAGE_REL_MIPS_LITERAL		0x0007	//Same as IMAGE_REL_MIPS_GPREL.
#define IMAGE_REL_MIPS_SECTION		0x000A	//The 16-bit section index of the section containing the target. This is used to support debugging information.
#define IMAGE_REL_MIPS_SECREL		0x000B	//The 32-bit offset of the target from the beginning of its section. This is used to support debugging information as well as static thread local storage.
#define IMAGE_REL_MIPS_SECRELLO		0x000C	//The low 16 bits of the 32-bit offset of the target from the beginning of its section.
#define IMAGE_REL_MIPS_SECRELHI		0x000D	//The high 16 bits of the 32-bit offset of the target from the beginning of its section. A PAIR relocation must immediately follow this on. The SymbolTableIndex of the PAIR relocation contains a signed 16-bit displacement, which is added to the upper 16 bits taken from the location being relocated.
#define IMAGE_REL_MIPS_JMPADDR16	0x0010	//The low 26 bits of the targets virtual address. This supports the MIPS16 JAL instruction.
#define IMAGE_REL_MIPS_REFWORDNB	0x0022	//The targets 32-bit relative virtual address.
#define IMAGE_REL_MIPS_PAIR			0x0025	//This relocation is only valid when it immediately follows a REFHI or SECRELHI relocation. Its SymbolTableIndex contains a displacement and not an index into the symbol table.
// .Type  Alpha Processors
#define IMAGE_REL_ALPHA_ABSOLUTE	0x0000	//This relocation is ignored.
#define IMAGE_REL_ALPHA_REFLONG	0x0001	//The targets 32-bit virtual address. This fixup is illegal in a PE32+ image unless the image has been sandboxed by clearing the IMAGE_FILE_LARGE_ADDRESS_AWARE bit in the File Header.
#define IMAGE_REL_ALPHA_REFQUAD	0x0002	//The targets 64-bit virtual address.
#define IMAGE_REL_ALPHA_GPREL32	0x0003	//32-bit signed displacement of the target relative to the Global Pointer (GP) register.
#define IMAGE_REL_ALPHA_LITERAL	0x0004	//16-bit signed displacement of the target relative to the Global Pointer (GP) register.
#define IMAGE_REL_ALPHA_LITUSE	0x0005	//Reserved for future use.
#define IMAGE_REL_ALPHA_GPDISP	0x0006	//Reserved for future use.
#define IMAGE_REL_ALPHA_BRADDR	0x0007	//The 21-bit relative displacement to the target. This supports the Alpha relative branch instructions.
#define IMAGE_REL_ALPHA_HINT	0x0008	//14-bit hints to the processor for the target of an Alpha jump instruction.
#define IMAGE_REL_ALPHA_INLINE_REFLONG	0x0009	//The targets 32-bit virtual address split into high and low 16-bit parts. Either an ABSOLUTE or MATCH relocation must immediately follow this relocation. The high 16 bits of the target address are stored in the location identified by the INLINE_REFLONG relocation. The low 16 bits are stored four bytes later if the following relocation is of type ABSOLUTE or at a signed displacement given in the SymbolTableIndex if the following relocation is of type MATCH.
#define IMAGE_REL_ALPHA_REFHI	0x000A	//The high 16 bits of the targets 32-bit virtual address. Used for the first instruction in a two-instruction sequence that loads a full address. This relocation must be immediately followed by a PAIR relocations whose SymbolTableIndex contains a signed 16-bit displacement which is added to the upper 16 bits taken from the location being relocated.
#define IMAGE_REL_ALPHA_REFLO	0x000B	//The low 16 bits of the targets virtual address.
#define IMAGE_REL_ALPHA_PAIR	0x000C	//This relocation is only valid when it immediately follows a REFHI , REFQ3, REFQ2, or SECRELHI relocation. Its SymbolTableIndex contains a displacement and not an index into the symbol table.
#define IMAGE_REL_ALPHA_MATCH	0x000D	//This relocation is only valid when it immediately follows INLINE_REFLONG relocation. Its SymbolTableIndex contains the displacement in bytes of the location for the matching low address and not an index into the symbol table.
#define IMAGE_REL_ALPHA_SECTION	0x000E	//The 16-bit section index of the section containing the target.  This is used to support debugging information.
#define IMAGE_REL_ALPHA_SECREL	0x000F	//The 32-bit offset of the target from the beginning of its section. This is used to support debugging information as well as static thread local storage.
#define IMAGE_REL_ALPHA_REFLONGNB	0x0010	//The targets 32-bit relative virtual address.
#define IMAGE_REL_ALPHA_SECRELLO	0x0011	//The low 16 bits of the 32-bit offset of the target from the beginning of its section.
#define IMAGE_REL_ALPHA_SECRELHI	0x0012	//The high 16 bits of the 32-bit offset of the target from the beginning of its section. A PAIR relocation must immediately follow this on. The SymbolTableIndex of the PAIR relocation contains a signed 16-bit displacement which is added to the upper 16 bits taken from the location being relocated.
#define IMAGE_REL_ALPHA_REFQ3	0x0013	//The low 16 bits of the high 32 bits of the targets 64-bit virtual address. This relocation must be immediately followed by a PAIR relocations whose SymbolTableIndex contains a signed 32-bit displacement which is added to the 16 bits taken from the location being relocated. The 16 bits in the relocated location are shifted left by 32 before this addition.
#define IMAGE_REL_ALPHA_REFQ2	0x0014	//The high 16 bits of the low 32 bits of the targets 64-bit virtual address. This relocation must be immediately followed by a PAIR relocations whose SymbolTableIndex contains a signed 16-bit displacement which is added to the upper 16 bits taken from the location being relocated.
#define IMAGE_REL_ALPHA_REFQ1	0x0015	//The low 16 bits of the targets 64-bit virtual address.
#define IMAGE_REL_ALPHA_GPRELLO	0x0016	//The low 16 bits of the 32-bit signed displacement of the target relative to the Global Pointer (GP) register.
#define IMAGE_REL_ALPHA_GPRELHI	0x0017	//The high 16 bits of the 32-bit signed displacement of the target relative to the Global Pointer (GP) register.
// .Type  IBM PowerPC Processors
#define IMAGE_REL_PPC_ABSOLUTE	0x0000	//This relocation is ignored.
#define IMAGE_REL_PPC_ADDR64	0x0001	//The targets 64-bit virtual address.
#define IMAGE_REL_PPC_ADDR32	0x0002	//The targets 32-bit virtual address.
#define IMAGE_REL_PPC_ADDR24	0x0003	//The low 24 bits of the targets virtual address. This is only valid when the target symbol is absolute and can be sign extended to its original value.
#define IMAGE_REL_PPC_ADDR16	0x0004	//The low 16 bits of the targets virtual address.
#define IMAGE_REL_PPC_ADDR14	0x0005	//The low 14 bits of the targets virtual address. This is only valid when the target symbol is absolute and can be sign extended to its original value.
#define IMAGE_REL_PPC_REL24		0x0006	//A 24-bit PC-relative offset to the symbols location.
#define IMAGE_REL_PPC_REL14		0x0007	//A 14-bit PC-relative offset to the symbols location.
#define IMAGE_REL_PPC_ADDR32NB	0x000A	//The targets 32-bit relative virtual address.
#define IMAGE_REL_PPC_SECREL	0x000B	//The 32-bit offset of the target from the beginning of its section. This is used to support debugging information as well as static thread local storage.
#define IMAGE_REL_PPC_SECTION	0x000C	//The 16-bit section index of the section containing the target. This is used to support debugging information.
#define IMAGE_REL_PPC_SECREL16	0x000F	//The 16-bit offset of the target from the beginning of its section. This is used to support debugging information as well as static thread local storage.
#define IMAGE_REL_PPC_REFHI		0x0010	//The high 16 bits of the targets 32-bit virtual address. Used for the first instruction in a two-instruction sequence that loads a full address. This relocation must be immediately followed by a PAIR relocations whose SymbolTableIndex contains a signed 16-bit displacement which is added to the upper 16 bits taken from the location being relocated.
#define IMAGE_REL_PPC_REFLO		0x0011	//The low 16 bits of the targets virtual address.
#define IMAGE_REL_PPC_PAIR		0x0012	//This relocation is only valid when it immediately follows a REFHI or SECRELHI relocation. Its SymbolTableIndex contains a displacement and not an index into the symbol table.
#define IMAGE_REL_PPC_SECRELLO	0x0013	//The low 16 bits of the 32-bit offset of the target from the beginning of its section.
#define IMAGE_REL_PPC_SECRELHI	0x0014	//The high 16 bits of the 32-bit offset of the target from the beginning of its section. A PAIR relocation must immediately follow this on. The SymbolTableIndex of the PAIR relocation contains a signed 16-bit displacement which is added to the upper 16 bits taken from the location being relocated.
#define IMAGE_REL_PPC_GPREL		0x0015	//16-bit signed displacement of the target relative to the Global Pointer (GP) register.
// .Type  Hitachi SuperH Processors
#define IMAGE_REL_SH3_ABSOLUTE		0x0000	//This relocation is ignored.
#define IMAGE_REL_SH3_DIRECT16		0x0001	//Reference to the 16-bit location that contains the virtual address of the target symbol.
#define IMAGE_REL_SH3_DIRECT32		0x0002	//The targets 32-bit virtual address.
#define IMAGE_REL_SH3_DIRECT8		0x0003	//Reference to the 8-bit location that contains the virtual address of the target symbol.
#define IMAGE_REL_SH3_DIRECT8_WORD	0x0004	//Reference to the 8-bit instruction that contains the effective 16-bit virtual address of the target symbol.
#define IMAGE_REL_SH3_DIRECT8_LONG	0x0005	//Reference to the 8-bit instruction that contains the effective 32-bit virtual address of the target symbol.
#define IMAGE_REL_SH3_DIRECT4		0x0006	//Reference to the 8-bit location whose low 4 bits contain the virtual address of the target symbol.
#define IMAGE_REL_SH3_DIRECT4_WORD	0x0007	//Reference to the 8-bit instruction whose low 4 bits contain the effective 16-bit virtual address of the target symbol.
#define IMAGE_REL_SH3_DIRECT4_LONG	0x0008	//Reference to the 8-bit instruction whose low 4 bits contain the effective 32-bit virtual address of the target symbol.
#define IMAGE_REL_SH3_PCREL8_WORD	0x0009	//Reference to the 8-bit instruction which contains the effective 16-bit relative offset of the target symbol.
#define IMAGE_REL_SH3_PCREL8_LONG	0x000A	//Reference to the 8-bit instruction which contains the effective 32-bit relative offset of the target symbol.
#define IMAGE_REL_SH3_PCREL12_WORD	0x000B	//Reference to the 16-bit instruction whose low 12 bits contain the effective 16-bit relative offset of the target symbol.
#define IMAGE_REL_SH3_STARTOF_SECTION	0x000C	//Reference to a 32-bit location that is the virtual address of the symbols section.
#define IMAGE_REL_SH3_SIZEOF_SECTION	0x000D	//Reference to the 32-bit location that is the size of the symbols section.
#define IMAGE_REL_SH3_SECTION		0x000E	//The 16-bit section index of the section containing the target. This is used to support debugging information.
#define IMAGE_REL_SH3_SECREL		0x000F	//The 32-bit offset of the target from the beginning of its section. This is used to support debugging information as well as static thread local storage.
#define IMAGE_REL_SH3_DIRECT32_NB	0x0010	//The targets 32-bit relative virtual address.
// .Type  ARM Processors
#define IMAGE_REL_ARM_ABSOLUTE	0x0000	//This relocation is ignored.
#define IMAGE_REL_ARM_ADDR32	0x0001	//The targets 32-bit virtual address.
#define IMAGE_REL_ARM_ADDR32NB	0x0002	//The targets 32-bit relative virtual address.
#define IMAGE_REL_ARM_BRANCH24	0x0003	//The 24-bit relative displacement to the target. 
#define IMAGE_REL_ARM_BRANCH11	0x0004	//Reference to a subroutine call, consisting of two 16-bit instructions with 11-bit offsets.
#define IMAGE_REL_ARM_SECTION	0x000E	//The 16-bit section index of the section containing the target. This is used to support debugging information.
#define IMAGE_REL_ARM_SECREL	0x000F	//The 32-bit offset of the target from the beginning of its section. This is used to support debugging information as well as static thread local storage.

#define COFF_AUXILIARYTABLE_SIZE 18
struct  COFF_AUXILIARYTABLE,1
{
	unsigned int  TagIndex;
	unsigned int  TotalSize;
	unsigned int  PointerToLinenumber;
	unsigned int  PointerToNextFunction;
	unsigned word Unused;
}
#define COFF_SYMBOLTABLE_SIZE 18
struct  COFF_SYMBOLTABLE,1 // 5.4. COFF Symbol Table
{
	dstring Name[0];     // union 1: name[8] or: if Zeroes=0 then name is in COFF_STRINGTABLE+Offset
	unsigned int  Zeroes;// union 1
	unsigned int  Offset;// union 1
	unsigned int  Value; // sapero: offset to data relative to data
	         word SectionNumber; // -2:debug, -1:absolute, 0-external, >0: COFF_SECTIONTABLE.name
	unsigned word Type;
	unsigned byte StorageClass; // IMAGE_SYM_CLASS enumeration
	unsigned byte NumberOfAuxSymbols;
	// Auxiliary Symbol Records here; no! not here!
	// COFF_STRINGTABLE
}
// .StorageClass
#define IMAGE_SYM_CLASS_END_OF_FUNCTION		0xFFFFFFFF
#define IMAGE_SYM_CLASS_NULL				0
#define IMAGE_SYM_CLASS_AUTOMATIC			1
#define IMAGE_SYM_CLASS_EXTERNAL			2 // extern or global (global if COFF_SYMBOLTABLE.SectionNumber>0 [or <>0 ??])
#define IMAGE_SYM_CLASS_STATIC				3
#define IMAGE_SYM_CLASS_REGISTER			4
#define IMAGE_SYM_CLASS_EXTERNAL_DEF		5
#define IMAGE_SYM_CLASS_LABEL				6
#define IMAGE_SYM_CLASS_UNDEFINED_LABEL		7
#define IMAGE_SYM_CLASS_MEMBER_OF_STRUCT	8
#define IMAGE_SYM_CLASS_ARGUMENT			9
#define IMAGE_SYM_CLASS_STRUCT_TAG			10
#define IMAGE_SYM_CLASS_MEMBER_OF_UNION		11
#define IMAGE_SYM_CLASS_UNION_TAG			12
#define IMAGE_SYM_CLASS_TYPE_DEFINITION		13
#define IMAGE_SYM_CLASS_UNDEFINED_STATIC	14
#define IMAGE_SYM_CLASS_ENUM_TAG			15
#define IMAGE_SYM_CLASS_MEMBER_OF_ENUM		16
#define IMAGE_SYM_CLASS_REGISTER_PARAM		17
#define IMAGE_SYM_CLASS_BIT_FIELD			18
#define IMAGE_SYM_CLASS_BLOCK				100
#define IMAGE_SYM_CLASS_FUNCTION			101
#define IMAGE_SYM_CLASS_END_OF_STRUCT		102
#define IMAGE_SYM_CLASS_FILE				103
#define IMAGE_SYM_CLASS_SECTION				104
#define IMAGE_SYM_CLASS_WEAK_EXTERNAL		105

struct COFF_STRINGTABLE,1
{
	unsigned int  size; // size of STRINGTABLE
	string   szarray[0];
}

// OBJHEADER
// SECTIONTABLE * OBJHEADER.NumberOfSections

// seek to OBJHEADER->OffsetToSymbolTable
// SYMBOLTABLE * OBJHEADER.NumberOfSymbols


//////////////////////////////
// LIB file

#define COFF_LIBSIGNATURE_SIZE 8
struct  COFF_LIBSIGNATURE,1
{
	dstring signature[8]; // !<arch>\x0A
}

#define COFF_MEMBERHEADER_SIZE 60
struct  COFF_MEMBERHEADER,1
{
	dstring name[16];  //Name of archive member, with a slash (/) appended to terminate the name.can be: name/  or / or //
	                   //  If the first character is a slash, the name has a special interpretation, as described below
	dstring date[12];  //Date and time the archive member was created: ASCII decimal
	                   //  representation of the number of seconds since 1/1/1970 UCT
	dstring id[6];     //ASCII decimal representation of the user ID
	dstring GroupID[6];//ASCII group representation of the group ID
	dstring Mode[8];   //ASCII octal representation of the members file mode
	dstring Size[10];  //ASCII decimal representation of the total size of the archive member,
	                   //  not including the size of the header
	dstring EndOfHeader[2];//The two bytes in the C string 13,13
}


//////////////////////////////

struct COFF_FIRSTMEMBER
{
	unsigned int NumberOfSymbols;
	unsigned int SymbolOffsets[0]; // size=NumberOfSymbols<<2 - file offsets to COFF_MEMBERHEADER
	string x[0];    // all global members in format string,NULL,string,NULL,...
}


struct COFF_SECONDMEMBER
{
	unsigned int  NumberOfMembers;
	unsigned int  MemberOffsets[0]; // size=NumberOfMembers<<2 - file offsets to COFF_MEMBERHEADER
	unsigned int  NumberOfSymbols;
	unsigned word Indices[0]; // Indices[NumberOfSymbols]
}

struct COFF_THIRDMEMBER
{
	string x[0]; // object filenames: obj_filename,NULL,obj_filename,NULL,...  the NULL can be not NULL :)
}
/*
COFF_LIBSIGNATURE

COFF_MEMBERHEADER name="/"
COFF_FIRSTMEMBER  size=COFF_MEMBERHEADER.Size

align=2
COFF_MEMBERHEADER name="/"                     this member can be not included!
COFF_SECONDMEMBER  size=COFF_MEMBERHEADER.Size

align=2
COFF_MEMBERHEADER name="//"
COFF_THIRDMEMBER  size=COFF_MEMBERHEADER.Size


COFF_MEMBERHEADER
OBJFILE_1


COFF_MEMBERHEADER
OBJFILE_2

...

COFF_MEMBERHEADER
OBJFILE_n
*/

// Machine Types
#define IMAGE_FILE_MACHINE_UNKNOWN		0x0
#define IMAGE_FILE_MACHINE_ALPHA		0x184
#define IMAGE_FILE_MACHINE_ARM			0x1c0
#define IMAGE_FILE_MACHINE_ALPHA64		0x284
#define IMAGE_FILE_MACHINE_I386			0x14c
#define IMAGE_FILE_MACHINE_IA64			0x200
#define IMAGE_FILE_MACHINE_M68K			0x268
#define IMAGE_FILE_MACHINE_MIPS16		0x266
#define IMAGE_FILE_MACHINE_MIPSFPU		0x366
#define IMAGE_FILE_MACHINE_MIPSFPU16	0x466
#define IMAGE_FILE_MACHINE_POWERPC		0x1f0
#define IMAGE_FILE_MACHINE_R3000		0x162
#define IMAGE_FILE_MACHINE_R4000		0x166
#define IMAGE_FILE_MACHINE_R10000		0x168
#define IMAGE_FILE_MACHINE_SH3			0x1a2
#define IMAGE_FILE_MACHINE_SH4			0x1a6
#define IMAGE_FILE_MACHINE_THUMB		0x1c2

// Characteristics
#define IMAGE_FILE_RELOCS_STRIPPED		0x0001
#define IMAGE_FILE_EXECUTABLE_IMAGE		0x0002
#define IMAGE_FILE_LINE_NUMS_STRIPPED	0x0004
#define IMAGE_FILE_LOCAL_SYMS_STRIPPED	0x0008
#define IMAGE_FILE_AGGRESSIVE_WS_TRIM	0x0010
#define IMAGE_FILE_LARGE_ADDRESS_AWARE	0x0020
#define IMAGE_FILE_16BIT_MACHINE		0x0040
#define IMAGE_FILE_BYTES_REVERSED_LO	0x0080
#define IMAGE_FILE_32BIT_MACHINE		0x0100
#define IMAGE_FILE_DEBUG_STRIPPED		0x0200
#define IMAGE_FILE_REMOVABLE_RUN_FROM_SWAP	0x0400
#define IMAGE_FILE_SYSTEM				0x1000
#define IMAGE_FILE_DLL					0x2000
#define IMAGE_FILE_UP_SYSTEM_ONLY		0x4000
#define IMAGE_FILE_BYTES_REVERSED_HI	0x8000








// from winnt.inc

//
// Section characteristics.
//
//      IMAGE_SCN_TYPE_REG                   0x00000000  // Reserved.
//      IMAGE_SCN_TYPE_DSECT                 0x00000001  // Reserved.
//      IMAGE_SCN_TYPE_NOLOAD                0x00000002  // Reserved.
//      IMAGE_SCN_TYPE_GROUP                 0x00000004  // Reserved.
#define IMAGE_SCN_TYPE_NO_PAD                0x00000008  // Reserved.
//      IMAGE_SCN_TYPE_COPY                  0x00000010  // Reserved.

#define IMAGE_SCN_CNT_CODE                   0x00000020  // Section contains code.
#define IMAGE_SCN_CNT_INITIALIZED_DATA       0x00000040  // Section contains initialized data.
#define IMAGE_SCN_CNT_UNINITIALIZED_DATA     0x00000080  // Section contains uninitialized data.

#define IMAGE_SCN_LNK_OTHER                  0x00000100  // Reserved.
#define IMAGE_SCN_LNK_INFO                   0x00000200  // Section contains comments or some other type of information.
//      IMAGE_SCN_TYPE_OVER                  0x00000400  // Reserved.
#define IMAGE_SCN_LNK_REMOVE                 0x00000800  // Section contents will not become part of image.
#define IMAGE_SCN_LNK_COMDAT                 0x00001000  // Section contents comdat.
//                                           0x00002000  // Reserved.
//      IMAGE_SCN_MEM_PROTECTED - Obsolete   0x00004000
#define IMAGE_SCN_NO_DEFER_SPEC_EXC          0x00004000  // Reset speculative exceptions handling bits in the TLB entries for this section.
#define IMAGE_SCN_GPREL                      0x00008000  // Section content can be accessed relative to GP
#define IMAGE_SCN_MEM_FARDATA                0x00008000
//      IMAGE_SCN_MEM_SYSHEAP  - Obsolete    0x00010000
#define IMAGE_SCN_MEM_PURGEABLE              0x00020000
#define IMAGE_SCN_MEM_16BIT                  0x00020000
#define IMAGE_SCN_MEM_LOCKED                 0x00040000
#define IMAGE_SCN_MEM_PRELOAD                0x00080000

#define IMAGE_SCN_ALIGN_1BYTES               0x00100000  //
#define IMAGE_SCN_ALIGN_2BYTES               0x00200000  //
#define IMAGE_SCN_ALIGN_4BYTES               0x00300000  //
#define IMAGE_SCN_ALIGN_8BYTES               0x00400000  //
#define IMAGE_SCN_ALIGN_16BYTES              0x00500000  // Default alignment if no others are specified.
#define IMAGE_SCN_ALIGN_32BYTES              0x00600000  //
#define IMAGE_SCN_ALIGN_64BYTES              0x00700000  //
#define IMAGE_SCN_ALIGN_128BYTES             0x00800000  //
#define IMAGE_SCN_ALIGN_256BYTES             0x00900000  //
#define IMAGE_SCN_ALIGN_512BYTES             0x00A00000  //
#define IMAGE_SCN_ALIGN_1024BYTES            0x00B00000  //
#define IMAGE_SCN_ALIGN_2048BYTES            0x00C00000  //
#define IMAGE_SCN_ALIGN_4096BYTES            0x00D00000  //
#define IMAGE_SCN_ALIGN_8192BYTES            0x00E00000  //
// Unused                                    0x00F00000
#define IMAGE_SCN_ALIGN_MASK                 0x00F00000

#define IMAGE_SCN_LNK_NRELOC_OVFL            0x01000000  // Section contains extended relocations.
#define IMAGE_SCN_MEM_DISCARDABLE            0x02000000  // Section can be discarded.
#define IMAGE_SCN_MEM_NOT_CACHED             0x04000000  // Section is not cachable.
#define IMAGE_SCN_MEM_NOT_PAGED              0x08000000  // Section is not pageable.
#define IMAGE_SCN_MEM_SHARED                 0x10000000  // Section is shareable.
#define IMAGE_SCN_MEM_EXECUTE                0x20000000  // Section is executable.
#define IMAGE_SCN_MEM_READ                   0x40000000  // Section is readable.
#define IMAGE_SCN_MEM_WRITE                  0x80000000  // Section is writeable.
